/* Alexa Gaming (via PubNub)
 * 
 * Created by Austin Wilson
 * 
 * Built to help bring Alexa into games
 * 
 * Notes:
 * For sessionAttributes, use handlerInput.attributesManager.getPersistentAttributes()
 * and handlerInput.attributesManager.setPersistentAttributes(attributes)
 * 
 * Add attribute manager to simplify communication between alexa and game
 */

var debug = false;
var PubNub = require('pubnub');
var pubnub = null;

module.exports = {
    setDebug(bool) {
        debug = bool;
    },

    setPubNub(publishKey, subscribeKey) {
        pubnub = new PubNub({
            publishKey : publishKey,
            subscribeKey : subscribeKey
        })
    },

    addChannelToGroup(channel, channelGroup) {
        return new Promise((resolve, reject) =>  {
            if(pubnub == null) {
                if(debug) console.log("Error", "You must call setPubNub()!");
                reject("You must call setPubNub()!");
            }
            pubnub.channelGroups.addChannels(
                {
                    channels: [channel],
                    channelGroup: channelGroup
                }, 
                function(status) {
                    if (status.error) {
                        if(debug) console.log("Error", status);
                        reject(status);
                    } else {
                        if(debug) console.log("Success", status);
                        resolve(status);
                    }
                }
            );
        });
    },

    publishEventSimple(message, channel) {
        return new Promise((resolve, reject) =>  {
            if(pubnub == null) {
                if(debug) console.log("Error", "You must call setPubNub()!");
                reject("You must call setPubNub()!");
            }
            // Create publish parameters
            // Format: {type: <VALUE>, message: <VALUE>}
            var params = {
                channel : channel,
                message : message
            }
            pubnub.publish(publishConfig, function(status, response) {
                if (status.error) {
                    if(debug) console.log("Error", status);
                    reject(status);
                } else {
                    if(debug) console.log("Success", response);
                    resolve(response);
                }
            })
        });
    },

    listChannels() {
        return new Promise((resolve, reject) =>  {
            if(pubnub == null) {
                if(debug) console.log("Error", "You must call setPubNub()!");
                reject("You must call setPubNub()!");
            }
            pubnub.channelGroups.listChannels(
                {
                    channelGroup: "myChannelGroup"
                }, 
                function (status, response) {
                    if (status.error) {
                        if(debug) console.log("Error", status);
                        reject(status);
                    } else {
                        if(debug) console.log("Success", response);
                        resolve(response);
                    }
                }
            );
        });
    },

    uniqueQueueGenerator() {
        var queue = this.listQueues().then((data) => {
            var channels = data.channels;
            var newChannel = this.makeId();
            if(channels) {
                while(channels.includes(newChannel)) {
                    newChannel = this.makeId();
                }
            }
            if(debug) console.log("New Channel Created: " + newChannel);
            return newChannel;
        }).catch((err) => {
            return err;
        });
        return queue;
    },

    makeId() {
        var text = "";
        var possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
        
        for (var i = 0; i < 5; i++) {
            text += possible.charAt(Math.floor(Math.random() * possible.length));
        }
        return text;
    }
}